package java.nio;

import java.util.function.IntConsumer;
import java.util.Spliterator;

class CharBufferSpliterator implements Spliterator.OfInt
{
    private final CharBuffer buffer;
    private int index;
    private final int limit;
    
    CharBufferSpliterator(CharBuffer charBuffer) {
        this(charBuffer, charBuffer.position(), charBuffer.limit());
    }
    
    CharBufferSpliterator(CharBuffer buffer, int n, int limit) {
        assert n <= limit;
        this.buffer = buffer;
        this.index = ((n <= limit) ? n : limit);
        this.limit = limit;
        return;
    }
    
    @Override
    public Spliterator.OfInt trySplit() {
        int index = this.index;
        int index2 = index + this.limit >>> 1;
        return (index >= index2) ? null : new CharBufferSpliterator(this.buffer, index, this.index = index2);
    }
    
    @Override
    public void forEachRemaining(IntConsumer intConsumer) {
        if (intConsumer == null) {
            throw new NullPointerException();
        }
        CharBuffer buffer = this.buffer;
        int i = this.index;
        int limit = this.limit;
        this.index = limit;
        while (i < limit) {
            intConsumer.accept((int)buffer.getUnchecked(i++));
        }
        return;
    }
    
    @Override
    public boolean tryAdvance(IntConsumer intConsumer) {
        if (intConsumer == null) {
            throw new NullPointerException();
        }
        if (this.index >= 0 && this.index < this.limit) {
            intConsumer.accept((int)this.buffer.getUnchecked(this.index++));
            return true;
        }
        return false;
    }
    
    @Override
    public long estimateSize() {
        return (long)(this.limit - this.index);
    }
    
    @Override
    public int characteristics() {
        return 16464;
    }
}
